@@ -2,6 +2,6 @@ from .chat_history import extract_chat_history
 from .dispatch_to_qa import send_to_qa, send_to_qa_async
 from .pubsub import process_pubsub
 from .special_commands import handle_special_commands, app_to_store, handle_files
-from .flask import register_qna_routes, create_app, VACRoutes
+from .flask import create_app, VACRoutes
 from .fastapi import register_qna_fastapi_routes, create_fastapi_app
 from .swagger import config_to_swagger
