@@ -191,55 +191,89 @@ def import_document_with_metadata_command(args):
 
 def search_command(args):
     """Handles the `discovery-engine search` subcommand (Data Store Chunks)."""
-    console.print(f"[cyan]Searching data store '{args.data_store_id}' for query: '{args.query}' (mode: chunks)[/cyan]")
+    console.print(f"[cyan]Searching data store '{args.data_store_id}' for query: '{args.query}' (mode: {args.content_search_spec_type})[/cyan]")
     try:
         client = DiscoveryEngineClient(
             project_id=args.project,
             data_store_id=args.data_store_id, # Target datastore
             location=args.location
         )
-        # This calls get_chunks which returns string or pager
-        results_data = client.get_chunks(
-            query=args.query,
-            # num_previous_chunks=args.num_previous_chunks, # Ensure these args are added to parser if needed
-            # num_next_chunks=args.num_next_chunks, # Ensure these args are added to parser if needed
-            page_size=args.page_size,
-            parse_chunks_to_string=args.parse_chunks_to_string,
-            serving_config=args.serving_config,
-            # data_store_ids=args.data_store_ids # Ensure these args are added to parser if needed
-        )
+
+        if args.content_search_spec_type == "chunks":
+            # This calls get_chunks which returns string or pager
+            results_data = client.get_chunks(
+                query=args.query,
+                # num_previous_chunks=args.num_previous_chunks, # Ensure these args are added to parser if needed
+                # num_next_chunks=args.num_next_chunks, # Ensure these args are added to parser if needed
+                page_size=args.page_size,
+                parse_chunks_to_string=args.parse_chunks_to_string,
+                serving_config=args.serving_config,
+                filter_str=args.filter,
+            )
+        elif args.content_search_spec_type == "documents":
+            results_data = client.get_documents(
+                query=args.query,
+                page_size=args.page_size,
+                parse_documents_to_string=args.parse_chunks_to_string,
+                serving_config=args.serving_config,
+                filter_str=args.filter,
+            )  
+        else:
+            raise ValueError("Invalid content_search_spec_type. Must be 'chunks' or 'documents'.")          
 
         if args.parse_chunks_to_string:
             console.print("\n[bold magenta]--- Combined Chunk String ---[/bold magenta]")
             console.print(results_data if results_data else "[yellow]No results found or error occurred.[/yellow]")
+        elif isinstance(results_data, str):
+            # Handle string result when parse_chunks_to_string is False but a string was returned anyway
+            console.print("\n[bold magenta]--- Results String ---[/bold magenta]")
+            console.print(results_data)
         elif results_data: # It's a pager object
-            console.print("\n[bold magenta]--- Individual Chunks ---[/bold magenta]")
-            chunk_count = 0
-            try:
-                 # Iterate through the pager returned by get_chunks
-                 for page in results_data.pages:
-                     if not hasattr(page, 'results') or not page.results: continue
-                     for result in page.results:
-                          # Ensure the result structure is as expected by get_chunks
-                          if hasattr(result, 'chunk'):
-                               chunk_count += 1
-                               console.print(f"\n[bold]Chunk {chunk_count}:[/bold]")
-                               # Use the client's formatter if available
-                               console.print(client.chunk_format(result.chunk))
-                          elif hasattr(result, 'document') and hasattr(result.document, 'chunks'):
-                               # Fallback if structure is different (e.g., document with chunks)
-                               for chunk in result.document.chunks:
-                                    chunk_count += 1
-                                    console.print(f"\n[bold]Chunk {chunk_count} (from doc {result.document.id}):[/bold]")
-                                    console.print(f"  Content: {getattr(chunk, 'content', 'N/A')}")
-                                    console.print(f"  Doc Name: {getattr(chunk, 'document_metadata', {}).get('name', 'N/A')}") # Example access
+            if args.content_search_spec_type == "chunks":
+                console.print("\n[bold magenta]--- Individual Chunks ---[/bold magenta]")
+                chunk_count = 0
+                try:
+                    # Iterate through the pager returned by get_chunks
+                    for page in results_data.pages:
+                        if not hasattr(page, 'results') or not page.results: continue
+                        for result in page.results:
+                            # Ensure the result structure is as expected by get_chunks
+                            if hasattr(result, 'chunk'):
+                                chunk_count += 1
+                                console.print(f"\n[bold]Chunk {chunk_count}:[/bold]")
+                                # Use the client's formatter if available
+                                console.print(client.chunk_format(result.chunk))
+                            elif hasattr(result, 'document') and hasattr(result.document, 'chunks'):
+                                # Fallback if structure is different (e.g., document with chunks)
+                                for chunk in result.document.chunks:
+                                        chunk_count += 1
+                                        console.print(f"\n[bold]Chunk {chunk_count} (from doc {result.document.id}):[/bold]")
+                                        console.print(f"  Content: {getattr(chunk, 'content', 'N/A')}")
+                                        console.print(f"  Doc Name: {getattr(chunk, 'document_metadata', {}).get('name', 'N/A')}") 
+                    if chunk_count == 0:
+                        console.print("[yellow]No chunks found in the results.[/yellow]")
 
-                 if chunk_count == 0:
-                     console.print("[yellow]No chunks found in the results.[/yellow]")
-
-            except Exception as page_err:
-                 console.print(f"[bold red]Error processing search results pager: {page_err}[/bold red]")
-                 console.print(f"[red]{traceback.format_exc()}[/red]")
+                except Exception as page_err:
+                    console.print(f"[bold red]Error processing search results pager: {page_err}[/bold red]")
+                    console.print(f"[red]{traceback.format_exc()}[/red]")
+            elif args.content_search_spec_type == "documents":
+                console.print("\n[bold magenta]--- Individual Documents ---[/bold magenta]")
+                doc_count = 0
+                try:
+                    # Iterate through the pager returned by get_documents
+                    for page in results_data.pages:
+                        if not hasattr(page, 'results') or not page.results: continue
+                        for result in page.results:
+                            if hasattr(result, 'document'):
+                                doc_count += 1
+                                console.print(f"\n[bold]Document {doc_count}:[/bold]")
+                                console.print(client.document_format(result.document))
+                    
+                    if doc_count == 0:
+                        console.print("[yellow]No documents found in the results.[/yellow]")
+                except Exception as page_err:
+                    console.print(f"[bold red]Error processing document results: {page_err}[/bold red]")
+                    console.print(f"[red]{traceback.format_exc()}[/red]")
         else:
             console.print("[yellow]No results found or error occurred.[/yellow]")
 
@@ -277,11 +311,25 @@ def search_by_id_and_or_date_command(args):
         elif results_data:
             console.print("\n[bold magenta]--- Individual Chunks (Filtered) ---[/bold magenta]")
             chunk_count = 0
-            # ... (pager iteration identical to search_command) ...
             try:
+                 # Iterate through the pager returned by get_chunks
                  for page in results_data.pages:
-                     # ... iterate results and chunks ...
-                      pass # Replace with actual iteration and printing
+                     if not hasattr(page, 'results') or not page.results: continue
+                     for result in page.results:
+                          # Ensure the result structure is as expected by get_chunks
+                          if hasattr(result, 'chunk'):
+                               chunk_count += 1
+                               console.print(f"\n[bold]Chunk {chunk_count}:[/bold]")
+                               # Use the client's formatter if available
+                               console.print(client.chunk_format(result.chunk))
+                          elif hasattr(result, 'document') and hasattr(result.document, 'chunks'):
+                               # Fallback if structure is different (e.g., document with chunks)
+                               for chunk in result.document.chunks:
+                                    chunk_count += 1
+                                    console.print(f"\n[bold]Chunk {chunk_count} (from doc {result.document.id}):[/bold]")
+                                    console.print(f"  Content: {getattr(chunk, 'content', 'N/A')}")
+                                    console.print(f"  Doc Name: {getattr(chunk, 'document_metadata', {}).get('name', 'N/A')}") # Example access
+
                  if chunk_count == 0:
                      console.print("[yellow]No chunks found in the filtered results.[/yellow]")
             except Exception as page_err:
@@ -306,9 +354,7 @@ def search_engine_command(args):
     try:
         client = DiscoveryEngineClient(
             project_id=args.project,
-            # data_store_id is required by __init__ but less relevant here.
-            # Provide a default or the primary one associated with the project/engine.
-            data_store_id=args.data_store_id_for_init,
+            engine_id=args.engine_id,
             location=args.location
         )
 
@@ -375,7 +421,7 @@ def search_engine_command(args):
                                         for source in citation.sources:
                                             # source is a CitationSource object with reference_index
                                             ref_idx = getattr(source, 'reference_index', 'N/A')
-                                            source_details.append(f"RefIdx:{ref_idx}") # Append details of each source
+                                            source_details.append(f"RefIdx:{ref_idx+1}") # Append details of each source
 
                                     source_info = ", ".join(source_details) if source_details else "No Source Info"
                                     start_idx = getattr(citation, 'start_index', 'N/A')
@@ -385,8 +431,8 @@ def search_engine_command(args):
                             references = page.summary.summary_with_metadata.references
                             if references:
                                 console.print("[bold cyan]References:[/bold cyan]")
-                                for ref in references:
-                                    console.print(f"  - Title: {getattr(ref, 'title', 'N/A')}, URI: {getattr(ref, 'uri', 'N/A')}") # Adjust based on actual reference structure
+                                for i, ref in enumerate(references):
+                                    console.print(f"  - {i+1} {ref}")
 
                         console.print("-" * 20)
 
@@ -399,35 +445,18 @@ def search_engine_command(args):
                             console.print(f"\n[bold]Result {i+1}:[/bold]")
                             doc = result.document
                             console.print(f"  ID: {doc.id}")
-                            console.print(f"  Name: {doc.name}")
                             # Display structData if present
                             if doc.struct_data:
-                                # Attempt 1: Preferred - Use proto-plus .to_json() if available
-                                if hasattr(doc.struct_data, 'to_json'):
-                                    console.print(f"[yellow]  to_json[/yellow]")
-                                    # Pass indent directly if the method supports it
-                                    try:
-                                        metadata_output = doc.struct_data.to_json(indent=2)
-                                    except TypeError: # In case to_json doesn't accept indent
-                                        metadata_output = doc.struct_data.to_json()
-                                        # Optionally re-parse and indent if needed, but often okay as is
-                                        # parsed_json = json.loads(metadata_output)
-                                        # metadata_output = json.dumps(parsed_json, indent=2)
-
-
-                                # Attempt 2: Use proto-plus .to_dict() then json.dumps
-                                elif hasattr(doc.struct_data, 'to_dict'):
-                                    console.print(f"[yellow]  to_dict[/yellow]")
-
-                                    struct_dict = doc.struct_data.to_dict()
-                                    metadata_output = json.dumps(struct_dict, indent=2)
-                                else:
-                                    try:
-                                        struct_dict = convert_composite_to_native(doc.struct_data)
-                                        metadata_output = struct_dict
-                                    except Exception as json_err:
-                                        console.print(f"[yellow]  Warning: Could not convert metadata Struct to JSON: {json_err}[/yellow]")
-                                        metadata_output = doc.struct_data
+                                try:
+                                    struct_dict = convert_composite_to_native(doc.struct_data)
+                                    metadata_output = struct_dict.get("structData", {})
+                                    title = struct_dict.get("title", "")
+                                    content = struct_dict.get("content", "")
+                                    console.print(f"  Title: {title}")
+                                    console.print(f"  Content: {content}")
+                                except Exception as json_err:
+                                    console.print(f"[yellow]  Warning: Could not convert metadata Struct to JSON: {json_err}[/yellow]")
+                                    metadata_output = doc.struct_data
                             console.print(f"  Metadata: {json.dumps(metadata_output, indent=2)}")
                             # Display Snippets if requested and available
                             if args.return_snippet and 'snippets' in doc.derived_struct_data:
@@ -512,13 +541,17 @@ def setup_discovery_engine_subparser(subparsers):
     import_doc_meta_parser.add_argument('--branch', default='default_branch', help='Target branch')
     import_doc_meta_parser.set_defaults(func=discovery_engine_command)
 
-    # --- Search Data Store (Chunks) subcommand ---
-    search_parser = discovery_engine_subparsers.add_parser('search', help='Search a datastore (fetches chunks)')
+    # --- Search Data Store (Chunks/Documents) subcommand ---
+    search_parser = discovery_engine_subparsers.add_parser('search', help='Search a datastore (fetches chunks or documents)')
     search_parser.add_argument('--query', required=True, help='The search query')
     search_parser.add_argument('--data-store-id', required=True, help='Data store ID to search')
     search_parser.add_argument('--page-size', type=int, default=10, help='Max results per page')
-    search_parser.add_argument('--parse-chunks-to-string', action='store_true', help='Output results as one formatted string')
+    search_parser.add_argument('--parse-chunks-to-string', action='store_true', help='Output results as one formatted string. Only applicable for "chunks"')
     search_parser.add_argument('--serving-config', default='default_config', help='Serving config ID for the data store')
+    search_parser.add_argument('--content_search_spec_type', default="chunks", help='"chunks" or "documents" depending on data store type')
+    search_parser.add_argument('--filter', help='filter for the search')
+
+
     # Add arguments for num_previous_chunks, num_next_chunks, data_store_ids if needed
     # search_parser.add_argument('--num-previous-chunks', type=int, default=3)
     # search_parser.add_argument('--num-next-chunks', type=int, default=3)
@@ -534,6 +567,8 @@ def setup_discovery_engine_subparser(subparsers):
     search_by_id_parser.add_argument('--page-size', type=int, default=10, help='Max results per page')
     search_by_id_parser.add_argument('--parse-chunks-to-string', action='store_true', help='Output results as one formatted string')
     search_by_id_parser.add_argument('--serving-config', default='default_config', help='Serving config ID')
+    search_by_id_parser.add_argument('--content_search_spec_type', default="chunks", help='"chunks" or "documents" depending on data store type')
+
     # Add arguments for num_previous_chunks, num_next_chunks, data_store_ids if needed
     # search_by_id_parser.add_argument('--num-previous-chunks', type=int, default=3)
     # search_by_id_parser.add_argument('--num-next-chunks', type=int, default=3)
@@ -544,8 +579,6 @@ def setup_discovery_engine_subparser(subparsers):
     search_engine_parser = discovery_engine_subparsers.add_parser('search-engine', help='Search a Discovery Engine (fetches documents/summary)')
     search_engine_parser.add_argument('--query', required=True, help='The search query')
     search_engine_parser.add_argument('--engine-id', required=True, help='Engine ID to search')
-    # Add data_store_id needed for client init, maybe make it optional if client handles it?
-    search_engine_parser.add_argument('--data-store-id-for-init', required=True, help='A primary data store ID associated with the project/engine (for client init)')
     search_engine_parser.add_argument('--serving-config-id', default='default_config', help='Serving config ID for the engine')
     search_engine_parser.add_argument('--collection-id', default='default_collection', help='Collection ID for the engine path')
     search_engine_parser.add_argument('--page-size', type=int, default=10, help='Max results per page')
