@@ -67,12 +67,19 @@ class DiscoveryEngineClient:
                 print(f"Document Name: {chunk_document_name}")
         ```
     """
-    def __init__(self, data_store_id, project_id, location="eu"):
+    def __init__(self, data_store_id=None, engine_id=None, project_id=None, location="eu"):
         if not discoveryengine:
             raise ImportError("Google Cloud Discovery Engine not available, install via `pip install sunholo[gcp]`")
+        
+        if project_id is None:
+            raise ValueError("Must specify project_id")
+        
+        if data_store_id is None and engine_id is None:
+            raise ValueError("Must specify at least one of data_store_id or engine_id")
 
         self.project_id = project_id
         self.data_store_id = data_store_id
+        self.engine_id = engine_id
         self.location = location
         client_options = (
             ClientOptions(api_endpoint=f"{location}-discoveryengine.googleapis.com")
@@ -206,21 +213,22 @@ class DiscoveryEngineClient:
         parse_chunks_to_string: bool = True,
         serving_config: str = "default_serving_config",
         data_store_ids: Optional[List[str]] = None,
+        filter_str:str=None
     ):
         """Retrieves chunks or documents based on a query.
 
         Args:
             query (str): The search query.
-            collection_id (str): The ID of the collection to search.
             num_previous_chunks (int, optional): Number of previous chunks to return for context (default is 3).
             num_next_chunks (int, optional): Number of next chunks to return for context (default is 3).
             page_size (int, optional): The maximum number of results to return per page (default is 10).
             parse_chunks_to_string: If True will put chunks in one big string, False will return object
             serving_config: The resource name of the Search serving config 
-            data_store_ids: If you want to search over many data stores, not just the one that was used to init the class. They should be of the format projects/{project}/locations/{location}/collections/{collection_id}/dataStores/{data_store_id}
+            data_store_ids: If you want to search over many data stores, not just the one that was used to init the class.
+                            They should be of the format projects/{project}/locations/{location}/collections/{collection_id}/dataStores/{data_store_id}
 
         Returns:
-            discoveryengine.SearchResponse: The search response object containing the search results.
+            discoveryengine.SearchResponse or str: The search response object or string of chunks.
 
         Example:
             ```python
@@ -231,51 +239,18 @@ class DiscoveryEngineClient:
             ```
         """
 
-        serving_config_path = self.search_client.serving_config_path(
-            self.project_id,
-            self.location,
-            self.data_store_id,
-            serving_config
-        )
-
-        search_request = discoveryengine.SearchRequest(
-            serving_config=serving_config_path,
+        return self.search_with_filters(
             query=query,
-            page_size=page_size, 
-            content_search_spec=discoveryengine.SearchRequest.ContentSearchSpec(
-                search_result_mode="CHUNKS", 
-                chunk_spec=discoveryengine.SearchRequest.ContentSearchSpec.ChunkSpec(
-                    num_previous_chunks=num_previous_chunks,
-                    num_next_chunks=num_next_chunks,
-                ),
-            ),
+            filter_str=filter_str,
+            num_previous_chunks=num_previous_chunks,
+            num_next_chunks=num_next_chunks,
+            page_size=page_size,
+            parse_chunks_to_string=parse_chunks_to_string,
+            serving_config=serving_config,
+            data_store_ids=data_store_ids,
+            content_search_spec_type="chunks"
         )
 
-        if data_store_ids:
-            search_request.data_store_specs = [
-                discoveryengine.SearchRequest.DataStoreSpec(
-                    data_store=self._search_data_store_path(data_store_id, serving_config=serving_config)
-                )
-                for data_store_id in data_store_ids
-            ]
-
-        try:
-            log.info(f"Discovery engine request: {search_request=}")
-            search_response = self.search_client.search(search_request)
-        except Exception as err:
-            log.warning(f"Error searching {search_request=} - no results found? {str(err)}")
-            search_response = []
-
-        if parse_chunks_to_string:
-
-            big_string = self.process_chunks(search_response)
-            log.info(f"Discovery engine chunks string sample: {big_string[:100]}")
-
-            return big_string
-        
-        log.info("Discovery engine response object")
-        return search_response
-
     async def async_get_chunks(
         self,
         query: str,
@@ -285,74 +260,34 @@ class DiscoveryEngineClient:
         parse_chunks_to_string: bool = True,
         serving_config: str = "default_serving_config",
         data_store_ids: Optional[List[str]] = None,
+        filter_str:str=None
     ):
-        """Retrieves chunks or documents based on a query.
+        """Asynchronously retrieves chunks or documents based on a query.
 
         Args:
             query (str): The search query.
-            collection_id (str): The ID of the collection to search.
             num_previous_chunks (int, optional): Number of previous chunks to return for context (default is 3).
             num_next_chunks (int, optional): Number of next chunks to return for context (default is 3).
             page_size (int, optional): The maximum number of results to return per page (default is 10).
             parse_chunks_to_string: If True will put chunks in one big string, False will return object
             serving_config: The resource name of the Search serving config 
-            data_store_ids: If you want to search over many data stores, not just the one that was used to init the class. They should be of the format projects/{project}/locations/{location}/collections/{collection_id}/dataStores/{data_store_id}
+            data_store_ids: If you want to search over many data stores, not just the one that was used to init the class.
+                            They should be of the format projects/{project}/locations/{location}/collections/{collection_id}/dataStores/{data_store_id}
 
         Returns:
-            discoveryengine.SearchResponse: The search response object containing the search results.
-
-        Example:
-            ```python
-            search_response = client.get_chunks('your query', 'your_collection_id')
-            for result in search_response.results:
-                for chunk in result.document.chunks:
-                    print(f"Chunk: {chunk.snippet}, document name: {chunk.document_name}")
-            ```
+            discoveryengine.SearchResponse or str: The search response object or string of chunks.
         """
-
-        serving_config_path = self.async_search_client.serving_config_path(
-            self.project_id,
-            self.location,
-            self.data_store_id,
-            serving_config
-        )
-
-
-        search_request = discoveryengine.SearchRequest(
-            serving_config=serving_config_path,
+        # Use async_search_with_filters with filter_str=None to perform a regular search
+        return await self.async_search_with_filters(
             query=query,
-            page_size=page_size, 
-            content_search_spec=discoveryengine.SearchRequest.ContentSearchSpec(
-                search_result_mode="CHUNKS", 
-                chunk_spec=discoveryengine.SearchRequest.ContentSearchSpec.ChunkSpec(
-                    num_previous_chunks=num_previous_chunks,
-                    num_next_chunks=num_next_chunks,
-                ),
-            ),
+            filter_str=filter_str,
+            num_previous_chunks=num_previous_chunks,
+            num_next_chunks=num_next_chunks,
+            page_size=page_size,
+            parse_chunks_to_string=parse_chunks_to_string,
+            serving_config=serving_config,
+            data_store_ids=data_store_ids
         )
-
-        if data_store_ids:
-            search_request.data_store_specs = [
-                discoveryengine.SearchRequest.DataStoreSpec(data_store=data_store_id)
-                for data_store_id in data_store_ids
-            ]
-
-        try:
-            log.info(f"Discovery engine request: {search_request=}")
-            search_response = self.async_search_client.search(search_request)
-        except Exception as err:
-            log.warning(f"Error searching {search_request=} - no results found? {str(err)}")
-            search_response = []
-
-        if parse_chunks_to_string:
-
-            big_string = await self.async_process_chunks(search_response)
-            log.info(f"Discovery engine chunks string sample: {big_string[:100]}")
-
-            return big_string
-        
-        log.info("Discovery engine response object")
-        return search_response
     
     def chunk_format(self, chunk):
 
@@ -376,8 +311,11 @@ class DiscoveryEngineClient:
         
         # Iterate through each result in the response
         for result in response.results:
-            chunk = result.chunk
-            chunk_metadata = chunk.ChunkMetadata
+            if hasattr(result, 'chunk'):      
+                chunk = result.chunk
+                chunk_metadata = chunk.ChunkMetadata
+            else:
+                log.warning("No chunk found in result")
 
             if hasattr(chunk_metadata, 'previous_chunks'):
                 # Process previous chunks
@@ -406,8 +344,11 @@ class DiscoveryEngineClient:
         
         # Iterate through each result in the response
         for result in response.results:
-            chunk = result.chunk
-            chunk_metadata = chunk.ChunkMetadata
+            if hasattr(result, 'chunk'):      
+                chunk = result.chunk
+                chunk_metadata = chunk.ChunkMetadata
+            else:
+                log.warning("No chunk found in result")
 
             if hasattr(chunk_metadata, 'previous_chunks'):
                 # Process previous chunks
@@ -425,6 +366,240 @@ class DiscoveryEngineClient:
         result_string = "\n".join(all_chunks)
 
         return result_string
+
+    def get_documents(
+        self,
+        query: str,
+        page_size: int = 10,
+        parse_documents_to_string: bool = True,
+        serving_config: str = "default_serving_config",
+        data_store_ids: Optional[List[str]] = None,
+        filter_str:str=None,
+        max_limit:int=None
+    ):
+        """Retrieves entire documents based on a query.
+
+        Args:
+            query (str): The search query.
+            page_size (int, optional): The maximum number of results to return per page (default is 10).
+            parse_documents_to_string: If True will put documents in one big string, False will return object
+            serving_config: The resource name of the Search serving config 
+            data_store_ids: If you want to search over many data stores, not just the one that was used to init the class.
+                            They should be of the format projects/{project}/locations/{location}/collections/{collection_id}/dataStores/{data_store_id}
+
+        Returns:
+            discoveryengine.SearchResponse or str: The search response object or string of documents.
+
+        Example:
+            ```python
+            search_response = client.get_documents('your query')
+            for result in search_response.results:
+                doc = result.document
+                print(f"Document: {doc.name}, Title: {doc.derived_struct_data.get('title')}")
+            ```
+        """
+        # Use search_with_filters with content_search_spec_type="documents" to get documents instead of chunks
+        return self.search_with_filters(
+            query=query,
+            filter_str=filter_str,
+            page_size=page_size,
+            parse_chunks_to_string=parse_documents_to_string,
+            serving_config=serving_config,
+            data_store_ids=data_store_ids,
+            content_search_spec_type="documents",
+            max_limit=max_limit
+        )
+
+    async def async_get_documents(
+        self,
+        query: str,
+        page_size: int = 10,
+        parse_documents_to_string: bool = True,
+        serving_config: str = "default_serving_config",
+        data_store_ids: Optional[List[str]] = None,
+        filter_str:str=None,
+        max_limit:int=None
+
+    ):
+        """Asynchronously retrieves entire documents based on a query.
+
+        Args:
+            query (str): The search query.
+            page_size (int, optional): The maximum number of results to return per page (default is 10).
+            parse_documents_to_string: If True will put documents in one big string, False will return object
+            serving_config: The resource name of the Search serving config 
+            data_store_ids: If you want to search over many data stores, not just the one that was used to init the class.
+                            They should be of the format projects/{project}/locations/{location}/collections/{collection_id}/dataStores/{data_store_id}
+
+        Returns:
+            discoveryengine.SearchResponse or str: The search response object or string of documents.
+        """
+        # Note: You'll need to update async_search_with_filters to handle content_search_spec_type
+        # as it doesn't currently have that parameter
+        return await self.async_search_with_filters(
+            query=query,
+            filter_str=filter_str,
+            page_size=page_size,
+            parse_chunks_to_string=parse_documents_to_string, 
+            serving_config=serving_config,
+            data_store_ids=data_store_ids,
+            content_search_spec_type="documents",
+            max_limit=max_limit
+
+        )
+
+    def document_format(self, document):
+        """Format a document for string output."""
+        # Extract useful fields from the document
+
+        document_id = document.id
+        document_name = document.name
+        
+        # Get content if available
+        content = ""
+        if hasattr(document, 'content') and document.content:
+            if hasattr(document.content, 'uri') and document.content.uri:
+                content = f"Content URI: {document.content.uri}\n"
+            if hasattr(document.content, 'mime_type') and document.content.mime_type:
+                content += f"Content Type: {document.content.mime_type}\n"
+        
+        # Get structured data if available
+        struct_data = ""
+        if hasattr(document, 'struct_data') and document.struct_data:
+            struct_data = f"Structured Data: {dict(document.struct_data)}\n"
+        
+        # Get derived structured data if available
+        derived_data = ""
+        if hasattr(document, 'derived_struct_data') and document.derived_struct_data:
+            derived_data = f"Derived Data: {dict(document.derived_struct_data)}\n"
+        
+        # Return formatted document string
+        return (
+            f"# Document: {document_id}\n"
+            f"Resource Name: {document_name}\n"
+            f"{content}"
+            f"{struct_data}"
+            f"{derived_data}"
+        )
+
+    def process_documents(self, response, max_limit:int=None):
+        """Process a search response containing documents into a formatted string."""
+        all_documents = []
+        result_count = 0
+        
+        # Check if the response contains results
+        if not response or not hasattr(response, 'results') or not response.results:
+            log.info(f'No results found in response: {response=}')
+            return []
+        
+        # Simpler approach: Just process the direct results from the response
+        # This handles the case where it's just a simple SearchResponse
+        for result in response.results:
+            if hasattr(result, 'document'):
+                document = result.document
+                all_documents.append(self.document_format(document))
+                result_count += 1
+                
+                # Check if we've reached max_limit
+                if max_limit is not None and result_count >= max_limit:
+                    log.info(f"Reached max_limit of {max_limit} results, stopping processing")
+                    break
+            else:
+                log.warning("No document found in result")
+        
+        # If there are pages, check gently without assuming too much about the structure
+        # Only process if we haven't already hit the max_limit
+        if (max_limit is None or result_count < max_limit) and hasattr(response, 'pages'):
+            try:
+                # For each page (after the first which we've already processed)
+                for page in list(response.pages)[1:]:  # Skip first page as we already processed it
+                    if hasattr(page, 'results'):
+                        for result in page.results:
+                            if hasattr(result, 'document'):
+                                document = result.document
+                                all_documents.append(self.document_format(document))
+                                result_count += 1
+                                
+                                # Check if we've reached max_limit
+                                if max_limit is not None and result_count >= max_limit:
+                                    log.info(f"Reached max_limit of {max_limit} results while processing pages")
+                                    break
+                            else:
+                                log.warning("No document found in result during page processing")
+                        
+                        # Break out of the page loop if we've reached max_limit
+                        if max_limit is not None and result_count >= max_limit:
+                            break
+            except Exception as e:
+                # Don't let page iteration issues break the whole function
+                log.warning(f"Error during page iteration, proceeding with results already collected: {e}")
+        
+        # Combine all documents into one long string
+        result_string = "\n\n".join(all_documents)
+        
+        return result_string
+
+    async def async_process_documents(self, response, max_limit:int=None):
+        """Process a search response containing documents into a formatted string asynchronously."""
+        all_documents = []
+        result_count = 0
+
+        # Check if the response contains results
+        if not response or not hasattr(response, 'results') or not response.results:
+            log.info(f'No results found in response: {response=}')
+            return []
+
+        # Simpler approach: Just process the direct results from the response
+        # This handles the case where it's just a simple SearchResponse
+        for result in response.results:
+            if hasattr(result, 'document'):
+                document = result.document
+                all_documents.append(self.document_format(document))
+                result_count += 1
+                
+                # Check if we've reached max_limit
+                if max_limit is not None and result_count >= max_limit:
+                    log.info(f"Reached max_limit of {max_limit} results, stopping processing")
+                    break
+            else:
+                log.warning("No document found in result")
+        
+        # If there are pages, check gently without assuming too much about the structure
+        # Only process if we haven't already hit the max_limit
+        if (max_limit is None or result_count < max_limit) and hasattr(response, 'pages'):
+            try:
+                # Since pages is an async iterator, we need special handling
+                first_page = True  # Flag to skip first page (already processed above)
+                async for page in response.pages:
+                    if first_page:
+                        first_page = False
+                        continue  # Skip first page as we already processed it
+                        
+                    if hasattr(page, 'results'):
+                        for result in page.results:
+                            if hasattr(result, 'document'):
+                                document = result.document
+                                all_documents.append(self.document_format(document))
+                                result_count += 1
+                                
+                                # Check if we've reached max_limit
+                                if max_limit is not None and result_count >= max_limit:
+                                    log.info(f"Reached max_limit of {max_limit} results while processing pages")
+                                    break
+                            else:
+                                log.warning("No document found in result during page processing")
+                        
+                        # Break out of the page loop if we've reached max_limit
+                        if max_limit is not None and result_count >= max_limit:
+                            break
+            except Exception as e:
+                # Don't let page iteration issues break the whole function
+                log.warning(f"Error during async page iteration, proceeding with results already collected: {e}")
+        
+        # Combine all documents into one long string
+        result_string = "\n\n".join(all_documents)
+        
+        return result_string
     
     def create_engine(self,
         engine_id: str, 
@@ -469,7 +644,7 @@ class DiscoveryEngineClient:
             operation = self.engine_client.create_engine(request=request)
         except AlreadyExists as err:
             log.info(f"Engine already exists: - {str(err)}")
-
+            self.engine_id = engine_id
             return engine_id
         
         log.info(f"Waiting for create vertex ai search operation to complete: {operation.operation.name}")
@@ -481,7 +656,7 @@ class DiscoveryEngineClient:
 
         # Handle the response
         log.info(f"{response=} {metadata=}")
-
+        self.engine_id = engine_id
         return operation.operation.name
 
     def _import_document_request(self,
@@ -686,7 +861,9 @@ class DiscoveryEngineClient:
                         num_previous_chunks=3, num_next_chunks=3, 
                         page_size=10, parse_chunks_to_string=True, 
                         serving_config="default_serving_config",
-                        data_store_ids: Optional[List[str]] = None):
+                        data_store_ids: Optional[List[str]] = None,
+                        content_search_spec_type="chunks",
+                        max_limit=None):
         """
         Searches with a generic filter string.
 
@@ -698,6 +875,8 @@ class DiscoveryEngineClient:
         Returns:
             discoveryengine.SearchResponse or str: The search response object or string of chunks.
         """
+        if max_limit is not None and max_limit < page_size:
+            page_size = max_limit
 
         serving_config_path = self.search_client.serving_config_path(
             self.project_id,
@@ -706,17 +885,26 @@ class DiscoveryEngineClient:
             serving_config
         )
 
+        if content_search_spec_type == "chunks":
+            content_search_spec=discoveryengine.SearchRequest.ContentSearchSpec(
+                            search_result_mode="CHUNKS", 
+                            chunk_spec=discoveryengine.SearchRequest.ContentSearchSpec.ChunkSpec(
+                                num_previous_chunks=num_previous_chunks,
+                                num_next_chunks=num_next_chunks,
+                            ),
+                        )
+        elif content_search_spec_type == "documents":
+            content_search_spec=discoveryengine.SearchRequest.ContentSearchSpec(
+                            search_result_mode="DOCUMENTS"
+                            )
+        else:
+            raise ValueError(f"Unknown content_search_spec_type={content_search_spec_type}")
+
         search_request = discoveryengine.SearchRequest(
             serving_config=serving_config_path,
             query=query,
             page_size=page_size, 
-            content_search_spec=discoveryengine.SearchRequest.ContentSearchSpec(
-                search_result_mode="CHUNKS", 
-                chunk_spec=discoveryengine.SearchRequest.ContentSearchSpec.ChunkSpec(
-                    num_previous_chunks=num_previous_chunks,
-                    num_next_chunks=num_next_chunks,
-                ),
-            ),
+            content_search_spec=content_search_spec,
             filter=filter_str # name:'ANY("king kong")'
         )
 
@@ -738,18 +926,30 @@ class DiscoveryEngineClient:
             return None
         
         if parse_chunks_to_string:
-            big_string = self.process_chunks(search_response)
-            log.info(f"Discovery engine chunks string sample: {big_string[:100]}")
-            return big_string
+            if content_search_spec_type=="chunks":
+                if parse_chunks_to_string:
+                    big_string = self.process_chunks(search_response)
+                    log.info(f"Discovery engine chunks string sample: {big_string[:100]}")
+
+                    return big_string
+                
+            elif content_search_spec_type=="documents":
+                big_string = self.process_documents(search_response, max_limit=max_limit)
+                log.info(f"Discovery engine documents string sample: {big_string[:100]}")
+
+                return big_string
         
         log.info("Discovery engine response object")
+
         return search_response
 
     async def async_search_with_filters(self, query, filter_str=None,
                             num_previous_chunks=3, num_next_chunks=3, 
                             page_size=10, parse_chunks_to_string=True, 
                             serving_config="default_serving_config",
-                            data_store_ids: Optional[List[str]] = None):
+                            data_store_ids: Optional[List[str]] = None,
+                            content_search_spec_type="chunks",
+                            max_limit=None):
         """
         Searches with a generic filter string asynchronously.
 
@@ -761,6 +961,8 @@ class DiscoveryEngineClient:
         Returns:
             discoveryengine.SearchResponse or str: The search response object or string of chunks.
         """
+        if max_limit is not None and max_limit < page_size:
+            page_size = max_limit
 
         serving_config_path = self.async_search_client.serving_config_path(
             self.project_id,
@@ -769,17 +971,26 @@ class DiscoveryEngineClient:
             serving_config
         )
 
+        if content_search_spec_type == "chunks":
+            content_search_spec=discoveryengine.SearchRequest.ContentSearchSpec(
+                            search_result_mode="CHUNKS", 
+                            chunk_spec=discoveryengine.SearchRequest.ContentSearchSpec.ChunkSpec(
+                                num_previous_chunks=num_previous_chunks,
+                                num_next_chunks=num_next_chunks,
+                            ),
+                        )
+        elif content_search_spec_type == "documents":
+            content_search_spec=discoveryengine.SearchRequest.ContentSearchSpec(
+                            search_result_mode="DOCUMENTS"
+                            )
+        else:
+            raise ValueError(f"Unknown content_search_spec_type={content_search_spec_type}")
+
         search_request = discoveryengine.SearchRequest(
             serving_config=serving_config_path,
             query=query,
             page_size=page_size, 
-            content_search_spec=discoveryengine.SearchRequest.ContentSearchSpec(
-                search_result_mode="CHUNKS", 
-                chunk_spec=discoveryengine.SearchRequest.ContentSearchSpec.ChunkSpec(
-                    num_previous_chunks=num_previous_chunks,
-                    num_next_chunks=num_next_chunks,
-                ),
-            ),
+            content_search_spec=content_search_spec,
             filter=filter_str # name:'ANY("king kong")'
         )
 
@@ -797,13 +1008,23 @@ class DiscoveryEngineClient:
         except Exception as e:
             log.info(f"No results {search_request.data_store_specs=}: {str(e)}")
             return None
-        
+            
         if parse_chunks_to_string:
-            big_string = await self.async_process_chunks(search_response)
-            log.info(f"Discovery engine chunks string sample: {big_string[:100]}")
-            return big_string
+            if content_search_spec_type=="chunks":
+                if parse_chunks_to_string:
+                    big_string = await self.async_process_chunks(search_response)
+                    log.info(f"Discovery engine chunks string sample: {big_string[:100]}")
+
+                    return big_string
+                
+            elif content_search_spec_type=="documents":
+                big_string = await self.async_process_documents(search_response, max_limit=max_limit)
+                log.info(f"Discovery engine documents string sample: {big_string[:100]}")
+
+                return big_string
         
         log.info("Discovery engine response object")
+        
         return search_response
 
     def search_by_objectId_and_or_date(self, query, objectId=None, date=None, **kwargs):
@@ -862,7 +1083,7 @@ class DiscoveryEngineClient:
     def search_engine(
         self,
         search_query: str,
-        engine_id: str,
+        engine_id: str = None,
         serving_config_id: str = "default_config",
         page_size: int = 10,
         return_snippet: bool = True,
@@ -890,7 +1111,7 @@ class DiscoveryEngineClient:
 
         Args:
             search_query: The user's search query string.
-            engine_id: The ID of the search engine to query.
+            engine_id: The ID of the search engine to query or uses class engine_id it init with.
             serving_config_id: The ID of the specific serving config for the engine.
             page_size: Maximum number of results per page.
             return_snippet: Whether to request snippets in the results.
@@ -961,13 +1182,20 @@ class DiscoveryEngineClient:
             log.error("Discovery Engine library not available at runtime.")
             return None
         
+        if engine_id:
+            self.engine_id = engine_id
+        
+        if engine_id is None and self.engine_id is None:
+            raise ValueError("Could not find self.engine_id")
+        
+        
         try:
             # Construct the serving config path for an ENGINE
             # Note: The client library path helper is for data stores/serving configs within them.
             # We need the path for an engine's serving config.
             serving_config_path = (
                 f"projects/{self.project_id}/locations/{self.location}/"
-                f"collections/{collection_id}/engines/{engine_id}/"
+                f"collections/{collection_id}/engines/{self.engine_id}/"
                 f"servingConfigs/{serving_config_id}"
             )
             log.info(f"Using Engine Serving Config Path: {serving_config_path}")
@@ -1029,22 +1257,22 @@ class DiscoveryEngineClient:
                 # Add other relevant fields like facet_specs if needed
             )
 
-            log.info(f"Searching engine '{engine_id}' with request: {request}")
+            log.info(f"Searching engine '{self.engine_id}' with request: {request}")
             response_pager = self.search_client.search(request)
-            log.info(f"Search successful for query '{search_query}' against engine '{engine_id}'.")
+            log.info(f"Search successful for query '{search_query}' against engine '{self.engine_id}'.")
             return response_pager
 
         except GoogleAPIError as e:
-            log.error(f"API error searching engine '{engine_id}': {e}")
+            log.error(f"API error searching engine '{self.engine_id}': {e}")
             return None
         except Exception as e:
-            log.error(f"Unexpected error searching engine '{engine_id}': {e}\n{traceback.format_exc()}")
+            log.error(f"Unexpected error searching engine '{self.engine_id}': {e}\n{traceback.format_exc()}")
             return None
 
     async def async_search_engine(
         self,
         search_query: str,
-        engine_id: str,
+        engine_id: str=None,
         serving_config_id: str = "default_config",
         page_size: int = 10,
         return_snippet: bool = True,
@@ -1077,6 +1305,12 @@ class DiscoveryEngineClient:
             An SearchAsyncPager object to iterate through results asynchronously,
             or None if an error occurs or the async client is not available.
         """
+        if engine_id:
+            self.engine_id = engine_id
+        
+        if engine_id is None and self.engine_id is None:
+            raise ValueError("Could not find self.engine_id")
+        
         if not self.async_search_client:
              log.error("Cannot call async_search_engine: Async client not initialized.")
              raise RuntimeError("Async client not initialized. Ensure class is instantiated within an async context.")
@@ -1085,7 +1319,7 @@ class DiscoveryEngineClient:
             # Construct the serving config path for an ENGINE (same as sync)
             serving_config_path = (
                 f"projects/{self.project_id}/locations/{self.location}/"
-                f"collections/{collection_id}/engines/{engine_id}/"
+                f"collections/{collection_id}/engines/{self.engine_id}/"
                 f"servingConfigs/{serving_config_id}"
             )
             log.info(f"Using Async Engine Serving Config Path: {serving_config_path}")
@@ -1138,16 +1372,16 @@ class DiscoveryEngineClient:
                 user_pseudo_id=user_pseudo_id,
             )
 
-            log.info(f"Async searching engine '{engine_id}' with request: {request}")
+            log.info(f"Async searching engine '{self.engine_id}' with request: {request}")
             response_pager = await self.async_search_client.search(request)
-            log.info(f"Async search successful for query '{search_query}' against engine '{engine_id}'.")
+            log.info(f"Async search successful for query '{search_query}' against engine '{self.engine_id}'.")
             return response_pager
 
         except GoogleAPIError as e:
-            log.error(f"Async API error searching engine '{engine_id}': {e}")
+            log.error(f"Async API error searching engine '{self.engine_id}': {e}")
             return None
         except Exception as e:
-            log.error(f"Async unexpected error searching engine '{engine_id}': {e}\n{traceback.format_exc()}")
+            log.error(f"Async unexpected error searching engine '{self.engine_id}': {e}\n{traceback.format_exc()}")
             return None
 
 # --- End of DiscoveryEngineClient class ---
\ No newline at end of file
